# plugin api
import plugin
import subprocess

import cyrusUtil

class CyrusPlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=users,dc=example,dc=com'
                self.nameAttr='uid'
                self.cyrusUser="cyrus"
                self.cyrusPassword="password"
                self.domain=None 


        def configure(self,conf):
                self.baseDn=conf.get("cyrus","baseDn")
                self.cyrusUser=conf.get("cyrus","username")
                self.cyrusPassword=conf.get("cyrus","password")
                self.cyrusServer=conf.get("cyrus","imapServer")
                self.domain=plugin.getConfig(conf,"cyrus","domain")
                self.quotaAttribute=plugin.getConfig(conf,"cyrus","quotaAttribute")
                self.reallyDelete=conf.get("cyrus","deleteMailboxes").lower()=="yes"


        def getUsername(self,dn):
                username=super(CyrusPlugin,self).getUsername(dn)
                if self.domain!=None:
                        username=username+"@"+self.domain
                return username


        def create(self,dn,attrs):
                if not self.filter(dn):
                        return

                username = self.getUsername(dn)
                #print "CyrusPlugin: create",username

                self.cyrusConnect()
                cyrusUtil.createUser(self.session,username)
                if self.quotaAttribute is not None and self.quotaAttribute in attrs:
                        cyrusUtil.setquota(self.session, username, self.get_quota(attrs))
                self.cyrusDisconnect()


        def get_quota(self,attrs):
                try:
                        quota = int(attrs[self.quotaAttribute][0])
                except:
                        quota = None

                return quota


        def update(self,dn,attrs):
                if not self.filter(dn):
                        return

                if self.quotaAttribute is None or self.quotaAttribute not in attrs:
                        return


                self.cyrusConnect()
                cyrusUtil.setquota(self.session, self.getUsername(dn), self.get_quota(attrs))
                self.cyrusDisconnect()


        def delete(self,dn):
                if not self.reallyDelete or not self.filter(dn):
                        return

                username = self.getUsername(dn)
                #print "deleting",username

                self.cyrusConnect()
                print cyrusUtil.deleteUser(self.session,username,self.cyrusUser)
                self.cyrusDisconnect()


        def rename(self,dn,attrs,dn2):
                if not self.filter(dn):
                        return

                oldUsername=self.getUsername(dn2)
                newUsername=self.getUsername(dn)

                self.cyrusConnect()
                cyrusUtil.renameUser(self.session,oldUsername,newUsername)
                self.cyrusDisconnect()


        def exists(self,dn,attrs):
                return False


        def cyrusConnect(self):
                self.session = cyrusUtil.login(self.cyrusServer,self.cyrusUser,self.cyrusPassword)


        def cyrusDisconnect(self):
                self.session.logout()

