# plugin api
import plugin
import subprocess
import opsiclient
import ldap
import re

optUseRpc=True

class OpsiPlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=computers,dc=example,dc=com'
                self.nameAttr='cn'
	        self.commandPrefix='sh -c '
	        self.clientNameSuffix="clients.example.com"
	        self.networksDn="ou=networks,dc=example,dc=com"
	        self.depotNameSuffix=None
                self.opsiRpcUrl="https://localhost:4447/rpc"
                self.useProfiles=False
                self.depotPrependNetwork=False
                self.profileFilter=None
                self.useTemplates=False
                self.usePrinters=False
                self.printersDn="ou=devices,dc=example,dc=com"
                self.ldapServer="localhost"
                self.adminDn="cn=admin,dc=example,dc=com"
                self.adminPassword="password"
                self.starttls=False
                self.printerPackage="printer"
                self.supplantKeyword="|>"
                self.productFilter=None


        def configure(self,conf):
		# print "OpsiPlugin.configure"
                self.debug = plugin.getConfig(conf, "opsi", "debug", False)
                self.baseDn=conf.get("opsi","baseDn")
                try:
                        self.commandPrefix=conf.get("opsi","commandPrefix")+" "
                except:
                        pass

		self.clientNameSuffix=conf.get("opsi","clientNameSuffix")
		self.depotNameSuffix=conf.get("opsi","depotName")
                self.depotPrependNetwork=conf.getboolean("opsi","depotPrependNetwork")
                self.useProfiles=conf.getboolean("opsi","useProfiles")
		self.networksDn=conf.get("opsi","networksDn")
                self.opsiRpcUrl=conf.get("opsi","rpcUrl")
                self.usePrinters=conf.getboolean("opsi","usePrinters")

		if self.useProfiles:
                        self.profilesDn=conf.get("opsi","profilesDn")
                        self.useTemplates=conf.getboolean("opsi","useTemplates")
                        try:
                                self.profileFilter=conf.get("opsi","profileFilter")
                        except:
                                pass
                        self.productFilter=plugin.getConfig(conf,"opsi", "productFilter", None)

                if self.usePrinters:
                        self.printersDn=conf.get("opsi","printersDn")
                        self.adminDn=conf.get("ldap","bindDn")
                        self.adminPassword=conf.get("ldap","bindPassword")
                        self.ldapServer=conf.get("ldap","server")
                        try:
                                self.starttls=conf.getboolean("ldap","starttls")
                        except:
                                pass

        def create(self,dn,attrs):
		# print "OpsiPlugin.create"
                if not self.filter(dn):
                        return

                if not self.filterByAttributes(attrs):
                        return

                self.update(dn,attrs)

        def copyTemplate(self,dn,attrs):
                ipAddress=attrs["ipHostNumber"][0]
                macAddress=attrs["macAddress"][0]
                if attrs.has_key("description"):
                        description=attrs["description"][0]
                else:
                        description=""

                # decode bytes to unicode string
                description=description.decode('utf-8')

                if self.useProfiles:
                        computername=self.getComputerName(dn)
                        depotName,profileName=self.parseSeeAlso(attrs)

                        if not self.filterByProfile(profileName):
                                return
                
                        if not self.useTemplates:
                                return

                        rpc=opsiclient.OpsiRpc(self.opsiRpcUrl)
                        if rpc.exists(computername):
                                return
                        #rpc.clean(computername)
                        print "OpsiPlugin.copyTemplate",profileName,computername
                        rpc.copyClient(profileName+self.clientNameSuffix,computername, \
                                ipAddress,macAddress,depotName,description)

        def update(self,dn,attrs):
                if not self.filter(dn):
                        return

                print "OpsiPlugin.update",dn
                if not self.filterByAttributes(attrs):
                        return
                
                depotName,profileName=self.parseSeeAlso(attrs)

                if not self.filterByProfile(profileName):
                        if self.exists(dn,None):
                                print "OpsiPlugin.update: profile",profileName,"does not match, removing"
                                self.delete(dn)
                                return
                        else:
                                print "OpsiPlugin.update: profile",profileName,"does not match, ignoring"
                                return

                computername=self.getComputerName(dn)

                mac_address=attrs["macAddress"][0]
                ip_address=attrs["ipHostNumber"][0]
                if attrs.has_key("description"):
                        description=attrs["description"][0]
                else:
                        description=""

                oldComputer = self.checkSupplant(dn,attrs)

                # print "createClient",computername,depotName
                rpc=opsiclient.OpsiRpc(self.opsiRpcUrl)
                if not self.useTemplates or rpc.exists(computername):
                        # use createClient to update
                        print " creating/updating",computername
                        # decode bytes to unicode string
                        description=description.decode('utf-8')
                        rpc.createClient(computername,None,description, \
                                None,mac_address,ip_address)
                else:
                        print " creating",computername,"from template"
                        self.copyTemplate(dn,attrs)

                if oldComputer is not None:
                        oldComputer += self.clientNameSuffix
                        self.copyProducts(oldComputer, computername, rpc, self.productFilter)

                rpc.clientSetDepot(computername,depotName)
                if self.usePrinters:
                        printers=self.findPrinters(dn,attrs)
                        parameters={}
                        i=1
                        for p in printers:
                                name,ip,queue,driver=p
                                print " configuring printer",name
                                prefix="flag_printer"+str(i)
                                parameters[prefix]=driver
                                parameters[prefix+"_name"]=queue
                                parameters[prefix+"_ip"]=ip
                                i=i+1

                        rpc.setProductPropertiesOnClient(computername, self.printerPackage, parameters)

                if oldComputer is not None:
                        self.log("deleting %s" % oldComputer)
                        rpc.deleteClient(oldComputer)

        def checkSupplant(self,dn,attrs):
                if "description" in attrs:
                        d = attrs['description'][0]
                        i = d.rfind(self.supplantKeyword)
                        if i >= 0:
                                self.log("checkSupplant found keyword")
                                cleanedDescription = d[:i]
                                attrs['description'][0] = cleanedDescription
                                supplantOrigin = d[i + len(self.supplantKeyword):]
                                self.log("checkSupplant cleanedDescription " + cleanedDescription)
                                if cleanedDescription == '':
                                        mod_attrs = [(ldap.MOD_DELETE, 'description', d)]
                                else:
                                        mod_attrs = [(ldap.MOD_REPLACE, 'description', cleanedDescription)]

                                c = self.getLdapConnection()

                                c.modify_s(dn, mod_attrs)

                                result = c.search_s(self.baseDn, ldap.SCOPE_SUBTREE, 'cn=%s' % supplantOrigin)
                                self.log("supplanting %s with %s" % (supplantOrigin, attrs['cn'][0]))
                                if len(result)>0:
                                        originDn = result[0][0]
                                        c.delete_s(originDn)
                                else:
                                        self.log("%s not found" % supplantOrigin)

                                self.disconnectLdap(c)
                                
                                return supplantOrigin
                return None


        def copyProducts(self, oldComputer, newComputer, rpc, productFilter):
                self.log("copyProducts from %s to %s" % (oldComputer,newComputer))
                oldProducts = rpc.getProductOnClient(oldComputer)
                newProducts = []
                
                for p in oldProducts:
                        if productFilter is not None and not re.match(productFilter,p['productId'],re.IGNORECASE):
                                continue

                        self.log("found product %s" % p['productId'])
                        p['clientId'] = newComputer
                        p['installationStatus'] = 'not_installed'
                        p['actionRequest'] = 'setup'

                        newProducts.append(p)

                rpc.rpc.productOnClient_createObjects(newProducts)

                for p in oldProducts:
                        productName = p['productId']
                        if productFilter is not None and not re.match(productFilter,p['productId'],re.IGNORECASE):
                                continue
                        props = rpc.getClientProductProperty(oldComputer, productName)
                        clean_props = {}
                        for k,v in props.iteritems():
                                clean_props[k] = v.decode('utf-8')
                        self.log("copying %d product properties for %s to %s"  % (len(props), productName, newComputer))
                        try:
                                rpc.setProductPropertiesOnClient(newComputer, productName, clean_props)
                        except:
                                self.log("error setProductPropertiesOnClient")


        def getLdapConnection(self):
                c = ldap.initialize(self.ldapServer)
                if self.starttls:
                        c.start_tls_s()
                c.simple_bind_s(self.adminDn,self.adminPassword)
                return c

        def disconnectLdap(self,c):
                c.unbind()

        def findPrinters(self,dn,attrs):
                printers=[]
                printerNames=[]
                l=len(self.nameAttr)+1
                for n in attrs["seeAlso"]:
                        i=n.rfind(self.printersDn)
                        if i>l:
                               printerName=n[l:i-1]
                               printerNames.append(printerName)

                c = self.getLdapConnection()

                printerNames.sort()

                for p in printerNames:
                        filter="%s=%s" % (self.nameAttr,p)
                        results=c.search_s(self.printersDn,ldap.SCOPE_SUBTREE,filter,None)
                        if len(results)==0:
                                print "ERROR: printer",p,"not found in LDAP"
                                continue
                        resultDn,resultAttrs=results[0]
                        printerName=p
                        printerIpAddress=resultAttrs['ipHostNumber'][0]
                        description=resultAttrs['description'][0]
                        parts=description.split(';')
                        printerQueue=parts[2]
                        printerDriver=parts[3]
                        printers.append((printerName,printerIpAddress,printerQueue,printerDriver))

                self.disconnectLdap(c)

                return printers

        def delete(self,dn):
                if not self.filter(dn):
                        return
                print "OpsiPlugin.delete",dn
                rpc=opsiclient.OpsiRpc(self.opsiRpcUrl)
                rpc.deleteClient(self.getComputerName(dn))

        def rename(self,dn,attrs,dn2):
                if not self.filter(dn):
                        return

        def exists(self,dn,attrs):
                if not self.filter(dn):
                        return
                computername=self.getComputerName(dn)
                rpc=opsiclient.OpsiRpc(self.opsiRpcUrl)
                return rpc.exists(computername)

        def parseSeeAlso(self,attrs):
                depotName=self.depotNameSuffix
                profileName=None

                l=len(self.nameAttr)+1
                for n in attrs["seeAlso"]:
                        if self.depotPrependNetwork:
                                i=n.rfind(self.networksDn)
                                if i>l:
                                        depotName=n[l:i-1]+self.depotNameSuffix
                        if self.useProfiles:
                                i=n.rfind(self.profilesDn)
                                if i>l:
                                        profileName=n[l:i-1]
                return (depotName,profileName)

        def getComputerName(self,dn):
                return self.getUsername(dn) + self.clientNameSuffix

        def command(self,cmd):
                print cmd
                subprocess.call(self.commandPrefix+cmd,shell=True)

        def filterByAttributes(self,attrs):
                if not attrs.has_key("macAddress") or not attrs.has_key("ipHostNumber"):
                        return False
                return True

        def filterByProfile(self,profileName):
                if self.useProfiles:
                        if (profileName==None or profileName=="" or profileName=='none' or profileName==' - ') \
                        or (self.profileFilter and self.profileFilter not in profileName ):
                                return False
                return True

