# plugin api
import ldap
import plugin
import subprocess
import shlex
from string import Template

class OxGroupPlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=group,dc=example,dc=com'
                self.nameAttr='cn'
                self.oxAuth='-A oxadmin -P password -c 1'
                self.attributeMap={}
                self.commandPrefix='ssh root@ox'
                self.filterAttribute=None
                self.filterValue=None
                self.filterStrict=False
                self.filterInvert=False
                self.filterDelete=True
                self.reallyDelete=True
                self.blacklistFile="/etc/dassadmin/groups.blacklist"
                self.whitelistFile="/etc/dassadmin/groups.whitelist"
                self.blacklistPrefix=None
                self.whitelistPrefix=None
                self.uidMap={}
                self.usernameMap={}

        def configure(self,configuration):
                section="oxGroup"
                self.baseDn=configuration.get(section,"baseDn")
                self.oxAuth="-c 1 -A "+configuration.get("ox","adminUser") \
                        + " -P "+configuration.get("ox","adminPassword")
                self.commandPrefix=configuration.get("ox","commandPrefix")+" "
                try:
                        self.filterAttribute=configuration.get(section,"filterAttribute")
                        if self.filterAttribute.lower()=="none":
                                self.filterAttribute=None
                except:
                        pass
                self.whitelistPrefix = plugin.getConfig(configuration, section, "whitelistPrefix", None)
                self.blacklistPrefix = plugin.getConfig(configuration, section, "blacklistPrefix", None)
                if self.filterAttribute:
                        self.filterValue=configuration.get(section,"filterValue")
                        self.filterStrict=configuration.get(section,"filterStrict").strip().lower()=="yes"
                        self.filterInvert=configuration.get(section,"filterInvert").strip().lower()=="yes"

                self.reallyDelete=configuration.get(section,"reallyDelete").strip().lower()=="yes"

        def create(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                print"OxGroupPlugin.create",dn
                if not self.filterByAttribute(dn,attrs):
                        return
                                
                name = self.getName(dn)

                self.command("/opt/open-xchange/sbin/creategroup "+self.oxAuth+" "+self.constructOxArgs(dn,attrs))

        def update(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                if not self.filterByAttribute(dn,attrs):
                        if self.filterDelete:
                                if self.exists(dn,attrs):
                                        self.delete(dn)
                        return
                groupname = self.getName(dn)
                print "OxGroupPlugin.update",groupname
                self.getIdMap()
                if attrs.has_key('memberUid'):
                        members=attrs['memberUid']
                else:
                        members=[]
               
                if attrs.has_key('displayName'): 
                        displayName=attrs['displayName'][0]
                elif attrs.has_key('description'):
                        displayName=attrs['description'][0]
                else:
                        displayName=groupname

                wantedMemberIds=[]
                for m in members:
                        if self.uidMap.has_key(m):
                                wantedMemberIds.append(self.uidMap[m])
                        else:
                                print " user "+m+" does not exist in ox"

                oxMembers=sorted(self.getGroupMembers(groupname))
                membersToBeAdded=[]
                membersToBeRemoved=[]

                #print " wanted members",sorted(wantedMemberIds)
                #print " actual members",oxMembers
                for m in wantedMemberIds:
                        c=" "
                        if not m in oxMembers:
                                c="+"
                                membersToBeAdded.append(m)
                        print " "+c+self.usernameMap[m]+"("+m+")"

                for m in oxMembers:
                        if not self.usernameMap.has_key(m):
                                print " ?"+m
                                continue
                        if not m in wantedMemberIds:
                                membersToBeRemoved.append(m)
                                print " -"+self.usernameMap[m]+"("+m+")"

                memberArgs=self.constructMemberArgs(membersToBeAdded,membersToBeRemoved)
                cmd="/opt/open-xchange/sbin/changegroup "+self.oxAuth+" -n \"" + groupname +"\" -d \""+displayName+"\""+memberArgs

                print cmd
                self.command(self.quote(cmd))

        def delete(self,dn):
                if not self.filter(dn,None) or self.reallyDelete==False:
                        return

                name = self.getName(dn)
                attrs = { self.nameAttr: [ name ] }

                if not self.filterByAttribute(dn,attrs):
                        return

                self.command("/opt/open-xchange/sbin/deletegroup "+self.oxAuth+" -n \""+name+"\"")

        def rename(self,dn,attrs,dn2):
                if not self.filter(dn,attrs):
                        return
                if not self.filterByAttribute(dn,attrs):
                        return

                oldUsername=self.getName(dn2)
                newUsername=self.getName(dn)
                self.command("\"/opt/open-xchange/sbin/changegroup "+self.oxAuth+" -i " 
                        + "\$(/opt/open-xchange/sbin/listgroup "+self.oxAuth+" | awk '/^ *[0-9]+ +"+oldUsername+" / { print \$1 }')"
                        + " -n \\\""+newUsername
                        + "\\\"\"")

        def exists(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return False
                if not self.filterByAttribute(dn,attrs):
                        return
                # print dn,self.baseDn,self.getUsername(dn)
                return self.checkGroup(self.getName(dn))

        def checkGroup(self,username):
                cmd=self.commandPrefix+"/opt/open-xchange/sbin/listgroup "+self.oxAuth \
                  +" | egrep -q '^ *[0-9]+ +"+username+" '"
                print cmd
                try:
                        subprocess.check_call(cmd,shell=True)
                except subprocess.CalledProcessError:
                        return False

                return True
                        
        def command(self,cmd):
                print self.commandPrefix+cmd
                return subprocess.check_call(self.commandPrefix+cmd,shell=True)

        def command_output(self,cmd):
                print self.commandPrefix+cmd
                child=subprocess.Popen(self.commandPrefix+cmd,shell=True,stdout=subprocess.PIPE)
                pipe=child.stdout
                result=pipe.readlines()
                child.wait()
                if child.returncode!=0:
                        raise Exception("command returned failure")
                return result
                
        def quote(self,s):
                return s.replace('\\','\\\\').replace('"','\\"').replace('&','\\&')

        def constructMemberArgs(self,membersToBeAdded,membersToBeDeleted):
                result=""
                if membersToBeAdded!=[]:
                        result=result+" -a "
                        for m in membersToBeAdded:
                                result=result+m+","
                        result=result[:-1]

                if membersToBeDeleted!=[]:
                        result=result+" -r " 
                        for m in membersToBeDeleted:
                                result=result+m+","
                        result=result[:-1]
                return result

        def constructOxArgs(self,dn,attrs):
                name=attrs[self.nameAttr][0]
                description=name
                try:
                        description=attrs['description'][0]
                except:
                        pass
                result='-n \\\"%s\\\" -d \\\"%s\\\"' % (name,description)
                return result

        def getGroupMembers(self,name):
                cmd="/opt/open-xchange/sbin/listgroup "+self.oxAuth+" -s \""+name+"\" --csv | awk '/^\" *[0-9]+\",\""+name+"\"/ { print }'"
                result=self.command_output(self.quote(cmd))
                if result==None or len(result)<1:
                        return []
                result=result[0].strip()
                result=result[:-1]
                # print result
                i=result.rfind('"')+1
                result=result[i:] 
                # print result
                if result=='':
                        return []
                members=result.split(",")
                return members 

        def getIdMap(self):
                cmd="/opt/open-xchange/sbin/listuser "+self.oxAuth+" | awk '/^ *[0-9]+ / { print $1,$2 }'"
                lines=self.command_output(cmd)
                for l in lines:
                        columns=l.strip().split(" ")
                        if len(columns)==2:
                                uid,username=columns
                                self.uidMap[username]=uid
                                self.usernameMap[uid]=username
                        else:
                                print " cannot parse",columns

        def filterByAttribute(self,dn,attrs):
                # print "OxGroupPlugin getBlacklist",self.getBlacklist()
                group_name = attrs[self.nameAttr][0].lower()

                whitelist = self.getWhitelist()
                whitelisted = False

                if whitelist is not None and group_name.lower() in whitelist:
                        whitelisted = True

                if self.whitelistPrefix is not None and group_name.startswith(self.whitelistPrefix):
                        whitelisted = True

                if (whitelist is not None or self.whitelistPrefix is not None) \
                        and not whitelisted:
                                print "OxGroupPlugin: ignoring group not on whitelist/without prefix", group_name
                                return False

                if group_name.lower() in self.getBlacklist():
                        print "OxGroupPlugin: ignoring blacklisted group", group_name
                        return False

                if self.blacklistPrefix is not None and group_name.startswith(self.blacklistPrefix):
                        print "OxGroupPlugin: ignoring group with blacklist prefix", group_name
                        return False

                if self.filterAttribute!=None:
                        if attrs.has_key(self.filterAttribute): 
                                return (attrs[self.filterAttribute][0]==self.filterValue) ^ self.filterInvert
                        else:
                                return self.filterStrict==False
                return True

        def getBlacklist(self):
                r=[]
                try:
                        f=open(self.blacklistFile,"r")
                        r=f.readlines()
                        f.close()
                except:
                        pass
                return [ s.strip().lower() for s in r ]

        def getWhitelist(self):
                r=[]
                try:
                        f=open(self.whitelistFile,"r")
                        r=f.readlines()
                        f.close()
                except:
                        pass
                whitelist = [ s.strip().lower() for s in r if not s=='\n' ]
                if len(whitelist) == 0:
                        return None
                else:
                        return whitelist
