# plugin api
import ldap
import plugin
import subprocess
import shlex
from string import Template

class OxResourcePlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=oxresources,ou=dassadmin,dc=example,dc=com'
                self.nameAttr='cn'
                self.oxAuth='-A oxadmin -P password -c 1'
                self.commandPrefix='sh -c '

        def configure(self,configuration):
                section="ox"
                self.baseDn=configuration.get("oxResource","baseDn")
                self.oxAuth="-c 1 -A "+configuration.get(section,"adminUser") \
                        + " -P "+configuration.get(section,"adminPassword")
                self.commandPrefix=configuration.get(section,"commandPrefix")+" "

        def create(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                                
                username = self.getUsername(dn)

                self.command("/opt/open-xchange/sbin/createresource "+self.oxAuth+" "+self.constructOxArgs(dn,attrs))

        def update(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                username = self.getUsername(dn)
                if not self.checkResource(username):
                        self.create(dn,attrs)
                else:
                        self.command("/opt/open-xchange/sbin/changeresource "+self.oxAuth +" "+self.constructOxArgs(dn,attrs))

        def delete(self,dn):
                if not self.filter(dn,None):
                        return
                username = self.getUsername(dn)
                self.command("/opt/open-xchange/sbin/deleteresource "+self.oxAuth+" -n "+username)

        def rename(self,dn,attrs,dn2):
                if not self.filter(dn,attrs):
                        return

                oldUsername=self.getUsername(dn2)
                newUsername=self.getUsername(dn)
                self.command("\"/opt/open-xchange/sbin/changeresource "+self.oxAuth+" -i " 
                        + "\$(/opt/open-xchange/sbin/listresource "+self.oxAuth+" | awk '/^[0-9]+ +"+oldUsername+" / { print \$1 }')"
                        + " -n "+newUsername
                        + "\"")

        def exists(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return False

                return self.checkResource(self.getUsername(dn))

        def checkResource(self,name):
                cmd=self.commandPrefix+"/opt/open-xchange/sbin/listresource "+self.oxAuth \
                  +" | egrep -qi '^ *[0-9]+ +"+name+" '"
                print cmd
                try:
                        subprocess.check_call(cmd,shell=True)
                except subprocess.CalledProcessError:
                        return False

                return True

        def constructOxArgs(self,dn,attrs):
                name=attrs['cn'][0]
                displayName=attrs['displayName'][0]
                available=attrs['employeeType'][0]
                mail=attrs['mail'][0]
                if attrs.has_key('description'):
                        description=attrs['description'][0]
                else:
                        description=''

                result="-n \\\"%s\\\" -e %s -d \\\"%s\\\" -D \\\"%s\\\" -a %s" \
                        % (name,mail,displayName,description,available)
                return result

        def command(self,cmd):
                print self.commandPrefix+cmd
                subprocess.check_call(self.commandPrefix+cmd,shell=True)

        def quote(self,s):
                return s.replace('\\','\\\\\\\\').replace('"','\\\\\\"').replace('&','\\&')
