# This plugin creates Idmap entries
# for new groups.
# The idea is to prevent member servers
# from inventing their own group ids.

# plugin api
import plugin
import subprocess
import ldap
import ldap.modlist

class SambaIdMapPlugin(plugin.Plugin):
        def __init__(self):
                self.groupsDn='ou=groups,dc=example,dc=com'
                self.baseDn=self.groupsDn
                self.idmapDn='ou=idmap,dc=example,dc=com'
                self.nameAttr='cn'
                self.adminDn='cn=admin,dc=example,dc=com'
                self.adminPassword='linux'
                self.ldapServer='localhost'
                self.starttls=False

        def configure(self,conf):
                self.groupsDn=conf.get("samba","groupsDn")
                self.baseDn=self.groupsDn
                self.idmapDn=conf.get("samba","idmapDn")
                self.adminDn=conf.get("ldap","bindDn")
                self.adminPassword=conf.get("ldap","bindPassword")
                self.ldapServer=conf.get("ldap","server")
                try:
                        self.starttls=conf.getboolean("ldap","starttls")
                except:
                        pass

                # print __name__,"loaded"

        def create(self,dn,attrs):
                # print dn
                if not self.filter(dn,attrs):
                        return
                groupname = self.getGroupName(dn)
                try:
                        self.createIdMap(dn,attrs)
                except:
                        self.updateIdMap(dn,attrs)

        def update(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return
                groupname = self.getGroupName(dn)
                try:
                        self.updateIdMap(dn,attrs)
                except:
                        self.createIdMap(dn,attrs)

        def delete(self,dn):
                if not self.filter(dn,None):
                        return

        def rename(self,dn,attrs,dn2):
                if not self.filter(dn,attrs):
                        return
                oldGroupname=self.getGroupName(dn2)
                newGroupname=self.getGroupName(dn)

        def exists(self,dn,attrs):
                return False

        def createIdMap(self,dn,attrs):
                if not attrs.has_key('sambaSID'):
                        return
                sid=attrs['sambaSID'][0]
                gid=attrs['gidNumber'][0]
                dn='sambaSID='+sid+','+self.idmapDn

                c = ldap.initialize(self.ldapServer)
                if self.starttls:
                        c.start_tls_s()
                c.simple_bind_s(self.adminDn,self.adminPassword)

                a={}
                a['objectClass']=['sambaIdmapEntry','sambaSIDEntry']
                a['sambaSID']=[ sid ]
                a['gidNumber']=[ gid ]
                m=ldap.modlist.addModlist(a)
                c.add_s(dn,m)

                c.unbind()

        def updateIdMap(self,dn,attrs):
                if not attrs.has_key('sambaSID'):
                        return
                sid=attrs['sambaSID'][0]
                gid=attrs['gidNumber'][0]
                dn='sambaSID='+sid+','+self.idmapDn

                print "SambaIdMap: updating",dn

                c = ldap.initialize(self.ldapServer)
                if self.starttls:
                        c.start_tls_s()
                c.simple_bind_s(self.adminDn,self.adminPassword)
                m=ldap.modlist.modifyModlist(attrs,{'gidNumber': gid },ignore_oldexistent=1)
                c.modify_s(dn,m)

                c.unbind()

        def getGroupName(self,dn):
                i=dn.find(',')
                rdn=dn[:i].lower()
                rest=dn[i+1:].lower()
                if rest!=self.baseDn.lower():
                        return None

                name=rdn.split("=")[1]

                if name=="" or ".." in name or "/" in name:
                        raise Exception("invalid group name")
               
                return name

        def command(self,cmd):
                print cmd
                subprocess.call(self.commandPrefix+cmd,shell=True)
