# plugin api
import plugin
import subprocess
import ldap
import re
import jobqueue

class VmProvisioningPlugin(plugin.Plugin):
        def __init__(self):
                self.baseDn='ou=computers,dc=example,dc=com'
                self.nameAttr='cn'
	        self.commandPrefix='sh -c '
	        self.vmNamePrefix="clients.example.com"
                self.createCommand="create-vm"
                self.profileFilter=None
                self.ldapServer="localhost"
                self.adminDn="cn=admin,dc=example,dc=com"
                self.adminPassword="password"
                self.starttls=False
                self.removeProfilePrefix=False

        def configure(self,conf):
		# print "VmProvisioningPlugin.configure"
                section = "vmprovisioning"
                self.debug = plugin.getConfig(conf, section, "debug", False)
                self.removeProfilePrefix = plugin.getConfig(conf, section, "removeProfilePrefix", False)
                self.baseDn=conf.get(section,"baseDn")
                try:
                        self.commandPrefix=conf.get(section,"commandPrefix")+" "
                except:
                        pass

		self.vmNamePrefix=conf.get(section,"vmNamePrefix")
                self.createCommand=conf.get(section,"createCommand")

                self.profilesDn=conf.get(section,"profilesDn")
                self.profileFilter=conf.get(section,"profileFilter")

                self.adminDn=conf.get("ldap","bindDn")
                self.adminPassword=conf.get("ldap","bindPassword")
                self.ldapServer=conf.get("ldap","server")
                self.starttls=plugin.getConfig(conf, "ldap", "starttls", False)


        def create(self,dn,attrs):
		# print "VmProvisioningPlugin.create"
                if not self.filter(dn):
                        return

                if not self.filterByAttributes(attrs):
                        return

                profileName, vmTemplateName = self.getProfileName(dn, attrs)

                if not self.filterByProfile(profileName):
                        return

                mac = self.commandOutput(
                        self.createCommand + " " \
                        + vmTemplateName + " " \
                        + self.getVmName(dn, attrs))

                self.log("MAC address is %s" % mac)

                if mac is not None and mac != "":
                        self.updateMac(dn, attrs, mac.strip())
                        self.resync(dn)


        def update(self,dn,attrs):
                if not self.filter(dn):
                        return

                print "VmProvisioningPlugin.update",dn


        def updateMac(self,dn,attrs,mac):
                mod_attrs = [(ldap.MOD_REPLACE, 'macAddress', mac)]

                c = self.getLdapConnection()

                self.log("modifying macAddress attribute on %s" % dn)
                c.modify_s(dn, mod_attrs)

                self.disconnectLdap(c)
                                
                return


        def getProfileName(self, dn, attrs):
                profileName=None
                vmTemplateName=None

                l=len(self.nameAttr)+1
                for n in attrs["seeAlso"]:
                        i=n.rfind(self.profilesDn)
                        if i>l:
                                profileName=n[l:i-1]

                if profileName is not None:
                        vmTemplateName=profileName
                        if self.removeProfilePrefix and profileName.startswith(self.profileFilter):
                                vmTemplateName = profileName[len(self.profileFilter):]

                return profileName, vmTemplateName


        def resync(self, dn):
                self.log("queueing job U %s" % dn)
                jobqueue.queueJob("U\n%s" % dn)


        def getLdapConnection(self):
                c = ldap.initialize(self.ldapServer)
                if self.starttls:
                        c.start_tls_s()
                c.simple_bind_s(self.adminDn,self.adminPassword)
                return c


        def disconnectLdap(self,c):
                c.unbind()


        def delete(self,dn):
                if not self.filter(dn):
                        return
                print "VmProvisioningPlugin.delete",dn


        def rename(self,dn,attrs,dn2):
                if not self.filter(dn):
                        return


        def exists(self,dn,attrs):
                if not self.filter(dn):
                        return
                return True


        def getVmName(self,dn, attrs):
                return self.vmNamePrefix + attrs[self.nameAttr][0]


        def commandOutput(self,cmd):
                print cmd
                return subprocess.check_output(self.commandPrefix+" " + cmd,shell=True)


        def filterByAttributes(self,attrs):
                if not attrs.has_key("ipHostNumber"):
                        return False
                return True


        def filterByProfile(self,profileName):
                if (profileName==None or profileName=="" or profileName=='none' or profileName==' - ') \
                        or (self.profileFilter and self.profileFilter not in profileName ):
                                return False
                return True

