#!/usr/bin/python
import sys
import traceback
import ConfigParser
import ldap
import ldif
import ldap.modlist
import datetime
import SocketServer
import signal

import jobqueue
import plugin

configPath='/etc/dassadmin/ldapsync.conf'

pluginTimeout=50
ldapServer='ldap://localhost'
ldapBaseDn=None
ldapBindDn=None
ldapPassword=None
con=None
starttls=False

plugins = [ ]

configuration = None

class PluginTimeout(Exception):
        pass


def signalHandler(signum, frame):
        raise PluginTimeout()


def fetchFromLDAP(dn):
        global con
        filter="objectclass=*"
        results=None

        try:
                results=con.search_s(dn,ldap.SCOPE_BASE,filter,None)
        except ldap.SERVER_DOWN:
                # make one attempt to reconnect
                ldapConnect()
                results=con.search_s(dn,ldap.SCOPE_BASE,filter,None)
        
        dn=results[0][0]
        attrs=ldap.cidict.cidict(results[0][1])
        # print attrs
        return (dn,attrs)


def initializePlugins(configuration):
        for p in plugins:
                p.configure(configuration)


def processJob(command,dn,dn2):
        global pluginTimeout
        signal.signal(signal.SIGALRM, signalHandler)

        attrs = None

        if dn is not None and dn != '' and command != 'D':
                (dn,attrs)=fetchFromLDAP(dn)

        for p in plugins:
                signal.alarm(pluginTimeout)
                try:
                        if command == 'U':
                                p.update(dn,attrs)

                        elif command == 'C':
                                p.create(dn,attrs)

                        elif command == 'D':
                                p.delete(dn)

                        elif command == 'R':
                                p.rename(dn,attrs,dn2)
                        else:
                                raise 'invalid command '+command

                except PluginTimeout:
                        print "ERROR: plugin timeout %s %s %s" % (command, p.__class__.__name__, dn)
                        raise
                finally:
                        signal.alarm(0)


def configure(configuration):
        global ldapServer,ldapBaseDn,ldapBindDn,ldapPassword,plugins,pluginTimeout,starttls

        ldapServer=configuration.get("ldap","server")
        ldapBaseDn=configuration.get("ldap","baseDn")
        ldapBindDn=configuration.get("ldap","bindDn")
        ldapPassword=configuration.get("ldap","bindPassword")
        if configuration.has_option("ldap","starttls"):
                starttls=configuration.getboolean("ldap","starttls")
        pluginTimeout=int(plugin.getConfig(configuration,"ldapsync", "pluginTimeout", "50"))
        s=configuration.get("ldapsync","plugins")
        pluginNames=s.split(",")
        
        for n in pluginNames:
                n=n.strip()+"Plugin"
                p=__import__(n)
                constructor=getattr(p,n)
                plugins.append(constructor())


def resync(con,baseDn,filter="objectclass=*"):
        results=con.search_s(baseDn,ldap.SCOPE_SUBTREE,filter,None)
        for dn,attrs in results:
                #print dn
                for p in plugins:
                        exists=p.exists(dn,attrs)
                        #print p.__class__.__name__
                        if not exists:
                                #print " create"
                                p.create(dn,attrs)
                        else:
                                p.update(dn,attrs)


def processQueue():
        # process a job from the queue
        job = jobqueue.dequeueJob()

        if job != None:
                command=job[0].strip()
                dn=job[1].strip()

                if len(job)>2:
                        dn2=job[2].strip()
                else:
                        dn2=None
                try: 
                        processJob(command,dn,dn2)
                except Exception:
                        raise 

        return job != None


class handler(SocketServer.BaseRequestHandler):
        def handle(self):
                ldapConnect()
                count=0
                errors=0
                more=True
                while more:
                        try:
                                more=processQueue()
                        except Exception:
                                traceback.print_exc() 
                                errors=errors+1

                        if more:
                                count=count+1
                if count>0:
                        print datetime.datetime.now(),"queue status:",count,"jobs processed,",errors,"errors"
                ldapDisconnect()
                self.request.send(str(count)+" "+str(errors)+"\n")


def ldapConnect():
        global con
        # connect to ldap server
        con = ldap.initialize(ldapServer)
        if starttls:
                con.start_tls_s()
        con.simple_bind_s(ldapBindDn,ldapPassword)


def ldapDisconnect():
        global con 
        con.unbind()


if __name__ == '__main__':
        # process configuration file
        configuration = ConfigParser.ConfigParser()
        configuration.read(configPath)

        configure(configuration)
        initializePlugins(configuration)
        
        if len(sys.argv)>1:
                if sys.argv[1]=='resync':
                        ldapConnect()
                        dn=sys.argv[2]
                        filter=sys.argv[3]
                        resync(con,dn,filter)
                        sys.exit(0)
                elif sys.argv[1]=='flush':
                        ldapConnect()
                        while processQueue():
                                pass
                elif sys.argv[1]=='server':
                        host='127.0.0.1'
                        port=int(sys.argv[2])
                        SocketServer.TCPServer.allow_reuse_address=True
                        server = SocketServer.TCPServer((host, port), handler)
                        server.serve_forever()

        else:
                ldapConnect()
                processQueue()
                ldapDisconnect() 
                
