#!/usr/bin/python
# coding: utf8

import os
import sys
import csv
import ldap
import ConfigParser
import httplib,urllib
import base64
import random
import string
import smtplib
import subprocess
from string import Template

configPath="/opt/dass-it/ldapsync/ldapsync.conf"

printerKeyword="#printer"

connection=None
configuration=None
baseDn=None
webserver="localhost"
httpUser="admin"
httpPassword="password"
authHash=""
passwordLength=8

performUpdate=False
performDelete=False

def processRow(row):
        if row[0]=='NAME':
                return

        fields = [ "name",
                "ipaddress",
                "macaddress",
                "description",
                "networkName",
                "printerQueue",
                "printerDriver" ]

        i=0
        entry={}

        for f in fields:
                if row[i]!=None and row[i]!="":
                        entry[f]=row[i].strip()
                i = i +1

        if performUpdate:
                updatePrinter(entry)
        elif performDelete:
                deletePrinter(entry)
        else:
                createPrinter(entry)

def postRequest(url,entry):
        params=urllib.urlencode(entry)
        headers = {"Content-type": "application/x-www-form-urlencoded",
                   "Accept": "text/html;text/plain",
                   "Authorization" : "Basic %s" % authHash}
        c = httplib.HTTPSConnection(webserver)
        #print params
        c.request("POST", "/dassadmin/devices/"+url, params, headers)
        response = c.getresponse()
        data = response.read()
        c.close()
        return (response.status,data)

def createPrinter(entry):
        entry["description"]="%s;%s;%s;%s" % ( entry["description"], printerKeyword,
                entry["printerQueue"], entry["printerDriver"])

        status,data = postRequest("new-action.vm",entry)
        if status>=400:
                print "ERROR: creating",entry['name']
                print status,data

def updatePrinter(entry):
        entry["key"]=entry["name"]
        entry["isprinter"]="true"
        entry["save"]="true"
        
        status,data = postRequest("edit-action.vm",entry)
        if status>=400:
                print "ERROR: updating",entry['name']
                print status,data

def deletePrinter(entry):
        entry["key"]=entry["name"]
        entry["delete"]="true"
        status,data = postRequest("delete-action.vm",entry)
        if status>=400:
                print "ERROR: deleting",entry['name']
                print status,data

def parseFile(f):
        reader = csv.reader(f,delimiter=';')
        for row in reader:
                processRow(row)

if __name__ == "__main__":
        configuration = ConfigParser.ConfigParser()
        configuration.read(configPath)

        httpUser=configuration.get("importer","httpUser")
        httpPassword=configuration.get("importer","httpPassword")

        authHash = base64.encodestring('%s:%s' % (httpUser, httpPassword))[:-1]

        file=sys.argv[1]
        parseFile(open(file,"r"))

        headers = {
                   "Accept": "text/json;text/plain",
                   "Authorization" : "Basic %s" % authHash}

        c = httplib.HTTPSConnection(webserver)
        c.request("GET","/dassadmin/queueStatus.vm",None,headers)
        r = c.getresponse()
        data = r.read()
        c.close()
        print r.status,data
