#!/bin/sh

basedn=$1
admindn=$2
adminpassword=$3
domain=$4
sambadomain=$5

init_ldap=$6

ttysave=$(stty -g)

trap "stty $ttysave;exit 2" 2

sambasid=$(net getlocalsid | awk '{ print $6}')
if [ -z "$sambasid" ]
then
	echo Kein Samba gefunden.
	sambasid=S-1-5-21-1234567890-1234567890-1234567890	
fi

if [ "$1" = "ask" -o -z "$1" ]
then
	echo dassadmin-Konfiguration
	echo =============================
	echo
	echo "Folgende Informationen werden benötigt:"
	echo "  LDAP-Base-DN (dc=example,dc=com)"
	echo "  DN des Administrators (cn=admin,dc=example,dc=com)"
	echo "  Passwort des LDAP-Administrators"
	echo "  Internet-Domain (example.com)"
	echo "  Samba-Domain (EXAMPLE)"
	echo
	echo Erkannte Samba-SID: $sambasid
	echo
	echo "Bitte geben Sie die Informationen ein oder Abbrechen mit ^C"
	echo
	echo -n "       Base-DN: "
	read basedn
	echo -n "      Admin-DN: "
	read admindn
	while [ -z "$adminpassword" -o "$adminpassword" != "$pwd2" ]
	do
		echo -n "Admin-Password: "
		stty -echo
		read adminpassword
		echo 
		echo -n "  Wiederholung: "
		read pwd2
		echo
		stty echo
		if [ "$adminpassword" != "$pwd2" ]
		then
			echo "Passwörter stimmen nicht überein."
		fi
	done
	echo -n "        Domain: "
	read domain
	echo -n "  Samba-Domain: "
	read sambadomain
	echo
fi


if [ -z  "$basedn" -o -z "$admindn" -o -z "$adminpassword" -o -z "$domain" \
	-o  -z "$sambadomain" ]
then
	echo "Usage: $0 basedn admindn adminpassword domain sambadomain"
	exit 1
fi

getname()
{
        dn=$1
        echo "$dn" | sed 's/\([[:alnum:]]\+\)=\([-[:alnum:] ]\+\).*/\2/'
}

basename=$(getname $basedn)
adminname=$(getname $admindn)
adminpwhash=$(echo -n $adminpassword | slappasswd -T /dev/stdin -n | sed -e 's/\//\\\//g' )

tomcatdir=/var/lib/tomcat7
tomcat=tomcat7
if [ ! -d "$tomcatdir" ]
then
	tomcatdir=/srv/tomcat7
fi
if [ ! -d "$tomcatdir" ]
then
	tomcatdir=/srv/tomcat6
	tomcat=tomcat6
fi
if [ ! -d "$tomcatdir" ]
then
	tomcatdir=/srv/tomcat
	tomcat=tomcat
fi
if [ ! -d "$tomcatdir" ]
then
	echo Tomcat-Installation nicht gefunden, Abbruch.
	exit 2
fi

ldapdir=/etc/openldap
if [ ! -d "$ldapdir" ]
then
	ldapdir=/etc/ldap
fi

webinf=$tomcatdir/webapps/dassadmin/WEB-INF
etc=/etc/dassadmin

files="
	$webinf/tools.xml
	$webinf/velocity.properties
	$webinf/ldap-init.ldif
	$webinf/ldapsync.conf
	$webinf/ox2ldap.conf
	"

replacecmd="-e s/@@BASEDN@@/$basedn/g
	-e s/@@BASENAME@@/$basename/g
	-e s/@@ADMINNAME@@/$adminname/g
	-e s/@@ADMINDN@@/$admindn/g
	-e s/@@ADMINPASSWORD@@/$adminpassword/g
	-e s/@@ADMINPWHASH@@/$adminpwhash/g
	-e s/@@SAMBASID@@/$sambasid/g
	-e s/@@SAMBADOMAIN@@/$sambadomain/g
	-e s/@@DOMAIN@@/$domain/g"

cp $etc/ldapsync.conf $webinf/ldapsync.conf.template
cp $etc/ox2ldap.conf $webinf/ox2ldap.conf.template

for f in $files
do
	if [ -f $f ]
	then
		echo $f already exists, skipping
	else
		echo processing $f
		sed $replacecmd $f.template >$f
	fi
done

cp $webinf/ldapsync.conf $etc
cp $webinf/ox2ldap.conf $etc
rm $webinf/ldapsync.conf.template $webinf/ldapsync.conf
rm $webinf/ox2ldap.conf.template $webinf/ox2ldap.conf

if [ "$init_ldap" = "true" ]
then
	rm -rf $ldapdir/slapd.d
	sed -e "s|/etc/ldap/|$ldapdir/|" $replacecmd <$webinf/slapd.conf.template >$ldapdir/slapd.conf
	if [ ! -r $ldapdir/schema/samba.schema ]
	then
		cp $webinf/samba.schema $ldapdir/schema
	fi
fi

if [ ! -f "$webinf/web.xml" ]
then
	cp $webinf/web.xml.dist $webinf/web.xml
fi

if grep dassadmin /etc/$tomcat/tomcat-users.xml
then
	: ok
else
	cat >/etc/$tomcat/tomcat-users.xml <<EOF
<tomcat-users>
<role rolename="tomcat"/>
<role rolename="Administrators"/>
<role rolename="Users"/>
<user username="tomcat" roles="tomcat" password="tomcat"/>
<user username="dassadmin" roles="tomcat,Admin,Users" password="dassadmin"/>
</tomcat-users>
EOF
fi

service slapd stop
service ldap stop
slapadd <$webinf/ldap-init.ldif
slapindex
chown ldap: /var/lib/ldap/*

chkconfig slapd on
chkconfig ldap on
chkconfig $tomcat on
chkconfig apache2 on
chkconfig dassadmin on

